package com.stockr.server.sentiment;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.paralleldots.paralleldots.App;
import com.stockr.server.news.Article;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.core.task.TaskExecutor;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.lang.reflect.Array;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Future;
import java.util.concurrent.PriorityBlockingQueue;
import java.util.stream.Collectors;

@Service
public class SentimentService {
    App pd = new App("ksfXrGlQnbn5jrJQBiGvxS0gdFVI8oHm6S3ZAxGz9EM");
    ObjectMapper om = new ObjectMapper();

    public Sentiment getSentiment(String text) throws Exception {
        String json = pd.sentiment(text);
        Sentiment sentiment = om.readValue(json, Sentiment.class);
        return sentiment;
    }

    public ArrayList<Article> getAllSentiments(ArrayList<Article> articles) throws Exception {
        ArrayList<Article> sentimentArticles = new ArrayList<>();
        for (Article article : articles) {
            Sentiment sentiment = getSentiment(article.getDescription());
            article.setSentiment(sentiment);
            sentimentArticles.add(article);
        }
        return sentimentArticles;
    }

    @Bean
    public TaskExecutor exec() {
        ThreadPoolTaskExecutor executor = new ThreadPoolTaskExecutor();
        executor.setCorePoolSize(10);
        executor.setMaxPoolSize(10);
        executor.setQueueCapacity(500);
        executor.setThreadNamePrefix("GithubLookup-");
        executor.initialize();
        return executor;
    }

    @Async(value = "exec")
    public CompletableFuture<Sentiment> gs(String text) {
        return CompletableFuture.supplyAsync(() -> {
            String json = null;
            try {
                json = pd.sentiment(text);
            } catch (Exception e) {
                e.printStackTrace();
            }
            Sentiment sentiment = null;
            try {
                sentiment = om.readValue(json, Sentiment.class);
            } catch (IOException e) {
                e.printStackTrace();
            }
            return sentiment;
        });
    }

    public List<Sentiment> gas(ArrayList<Article> articles) throws Exception {
        System.out.println(System.currentTimeMillis());
        List<CompletableFuture<Sentiment>> sentimentArticles = articles.stream()
                .map(article -> gs(article.getDescription()))
                .collect(Collectors.toList());

        CompletableFuture<Void> allFutures = CompletableFuture.allOf(
                sentimentArticles.toArray(new CompletableFuture[sentimentArticles.size()])
        );

        CompletableFuture<List<Sentiment>> sentimentFutures = allFutures.thenApply(v -> {
            return sentimentArticles.stream().map(x -> x.join()).collect(Collectors.toList());
        });

        List<Sentiment> fin = sentimentFutures.get();
        System.out.println(System.currentTimeMillis());
        return fin;
    }
}
