package com.stockr.server.stocks;

import org.springframework.context.annotation.Bean;
import org.springframework.stereotype.Service;
import pl.zankowski.iextrading4j.api.stocks.BatchStocks;
import pl.zankowski.iextrading4j.api.stocks.Quote;
import pl.zankowski.iextrading4j.client.IEXTradingClient;
import pl.zankowski.iextrading4j.client.rest.manager.RestRequest;
import pl.zankowski.iextrading4j.client.rest.request.stocks.BatchStocksRequestBuilder;
import pl.zankowski.iextrading4j.client.rest.request.stocks.BatchStocksType;
import pl.zankowski.iextrading4j.client.rest.request.stocks.ListRequestBuilder;
import pl.zankowski.iextrading4j.client.rest.request.stocks.ListType;

import java.util.List;
import java.util.stream.Collectors;

@Service
public class StocksService {
    private final IEXTradingClient iex = IEXTradingClient.create();

    public Stocks getStocks(String ticker) {
        return new Stocks(getBatchFromTicker(ticker));
    }

    // https://api.iextrading.com/1.0/stock/market/list/gainers
    public List<Gainer> getGainers() {
        List<Quote> quotes = iex.executeRequest(new ListRequestBuilder()
                .withListType(ListType.GAINERS)
                .build());

        return quotes
                .stream()
                .map(Gainer::new)
                .collect(Collectors.toList());
    }

    private BatchStocks getBatchFromTicker(String ticker) {
        return iex.executeRequest(new BatchStocksRequestBuilder()
                .withSymbol(ticker)
                .addType(BatchStocksType.COMPANY)
                .addType(BatchStocksType.CHART)
                .addType(BatchStocksType.NEWS)
                .addType(BatchStocksType.QUOTE)
                .build());
    }
}
