package com.stockr.server.news;

import static org.mockito.Mockito.when;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.handler;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import org.junit.Ignore;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.servlet.MockMvc;

@RunWith(SpringRunner.class)
@WebMvcTest(NewsController.class)
public class NewsControllerTest {

    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private NewsService service;

    @Test
    public void news_ShouldUseMethod_getAllArticles() throws Exception {
        when(service.getNews("test")).thenReturn(new News());
        this.mockMvc.perform(get("/news?query=fb"))
                .andDo(print())
                .andExpect(handler().methodName("getAllArticles"));
    }

    @Test
    public void news_ShouldReturnJsonArray() throws Exception {
        when(service.getNews("test")).thenReturn(new News());
        this.mockMvc.perform(get("/news?query=fb"))
                .andDo(print())
                .andExpect(content().string("[]"));
    }

    @Test
    public void news_ShouldHaveBadRequest_WhenNoQueries() throws Exception {
        when(service.getNews("test")).thenReturn(new News());
        this.mockMvc.perform(get("/news"))
                .andDo(print())
                .andExpect(status().isBadRequest());
    }
}