package com.stockr.server.stocks;

import com.stockr.server.stocks.Stocks;
import com.stockr.server.stocks.StocksController;
import com.stockr.server.stocks.StocksService;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.servlet.MockMvc;
import pl.zankowski.iextrading4j.api.stocks.News;

import static org.mockito.Mockito.when;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.handler;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@RunWith(SpringRunner.class)
@WebMvcTest(StocksController.class)
public class StocksControllerTest {

    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private StocksService service;

    @Test
    public void stocks_ShouldUseMethod_getStocks() throws Exception {
        when(service.getStocks("test")).thenReturn(new Stocks());
        this.mockMvc.perform(get("/stocks?ticker=test"))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(handler().methodName("getStocks"));
    }

    @Test
    public void stocks_ShouldThrowBadRequest_GivenNoTicker() throws Exception {
        when(service.getStocks("test")).thenReturn(new Stocks());
        this.mockMvc.perform(get("/stocks"))
                .andDo(print())
                .andExpect(status().isBadRequest());
    }

    @Test
    public void gainers_ShouldUseMethod_getGainers() throws Exception {
        when(service.getStocks("test")).thenReturn(new Stocks());
        this.mockMvc.perform(get("/stocks/gainers"))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(handler().methodName("getGainers"));
    }
}
